#include <gtk/gtk.h>
#include <glade/glade.h>
#include <loudmouth/loudmouth.h>
#include <stdio.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>
#include "kf.h"
#include "jabber.h"
#include "archive.h"

static void kf_archive_real (LmMessage *msg, const gchar *jid, time_t stamp);

/* Puts a LmMessage into archive.
 * set yours to TRUE for outcoming messages */
void kf_archive_msg (LmMessage *msg, gboolean yours) {
	LmMessageNode *node;
	const gchar *jid;
	gchar *cjid;
	gchar *stamp;

	g_return_if_fail (msg->node);

//	g_print ("***\n%s\n***\n", lm_message_node_to_string (msg->node));
	if (yours)
		jid = lm_message_node_get_attribute (msg->node, "to");
	else
		jid = lm_message_node_get_attribute (msg->node, "from");

	g_return_if_fail (jid);

	cjid = g_strdup (jid);
	kf_jabber_jid_crop (cjid);

	stamp = g_strdup_printf ("%d", time (NULL));
	//stamp = g_strdup_printf ("%d", time (NULL));
	lm_message_node_set_attribute (msg->node, "kf:stamp", stamp);
	g_free (stamp);

	lm_message_node_set_attribute (msg->node, "kf:direction", yours?"out":"in");

	kf_archive_real (msg, cjid, time (NULL));

	g_free (cjid);
}

static void kf_archive_real (LmMessage *msg, const gchar *jid, time_t stamp) {
	FILE *f;
	const gchar *path;
	gchar *filename;
	struct tm *time;
	LmMessageSubType type;
	gchar *text;
	
	type = lm_message_get_sub_type (msg);
	
	/* Let's create archive directory, if it doesn't exist */
	if (type == LM_MESSAGE_SUB_TYPE_NORMAL) {
		path = kf_config_file ("archive/msgs");
	} else if (type == LM_MESSAGE_SUB_TYPE_CHAT) {
		path = kf_config_file ("archive/chats");
	} else if (type == LM_MESSAGE_SUB_TYPE_GROUPCHAT) {
		path = kf_config_file ("archive/muc");
	} else if (type == LM_MESSAGE_SUB_TYPE_HEADLINE) {
		path = kf_config_file ("archive/msgs");
	} else {
		path = kf_config_file ("archive/chats");
	}

	if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
		mkdir (path, 00753);
	}

	filename = g_strdup_printf ("%s/%s.xml", path, jid);

//	g_print ("path: %s, filename: %s\n", path, filename);
	time = localtime ((time_t *) &(stamp));
	if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
		f = fopen (filename, "ab");

		fputs ("<?xml version='1.0'?>\n", f);
		fputs ("<archive xmlns='http://jabber.org/protocol/archive'>\n", f);
		fprintf (f, "<item start=\"%04d%02d%02dT%02d:%02d:%02d\">\n",
			time->tm_year+1900, time->tm_mon+1, time->tm_mday,
			time->tm_hour, time->tm_min, time->tm_sec);
	} else {
		struct stat stat_buffer;
		stat (filename, &stat_buffer);

		f = fopen (filename, "ab");

		if (stat_buffer.st_mtime < kf_get_start_time ()) {
			/* File has not been modified during this session */
			foo_debug ("Creating new <item>\n");

			fputs ("</item>\n", f);
			fprintf (f, "<item start=\"%04d%02d%02dT%02d:%02d:%02d\">\n",
			time->tm_year+1900, time->tm_mon+1, time->tm_mday,
			time->tm_hour, time->tm_min, time->tm_sec);
		}
	}
	g_return_if_fail (f);


//	if (fseek (f, -20L, SEEK_END)) {
//		g_print ("AAAA");
//	}

	text = lm_message_node_to_string (msg->node);
	fputs (text, f);
//	fputs ("</archive>\n", f);
	g_free (text);

	fclose (f);

	g_free (filename);
}
