#include <gtk/gtk.h>
#include <stdio.h>
#include <ctype.h>
#include "kf.h"
#include "emoticons.h"

/* Animated emoticons */
//#define ANIM

typedef struct {
	gchar *name;
	gchar *filename;
	gchar *path;
	GdkPixbuf *pixbuf;
} KfEmoticon;

void kf_emoticons_insert_text (GtkTextBuffer *buffer, const gchar *text);
GList *kf_emoticons_load (const gchar *filename);
KfEmoticon *kf_emoticon_new (const gchar *name, const gchar *path, const gchar *file);
//void kf_emoticons_init (void);
static gboolean textview_clicked_cb (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
static void textview_popup_cb (GtkTextView *textview, GtkMenu *menu, gpointer user_data);
static void open_link_cb (GtkMenuItem *menuitem, gpointer user_data);

GList *kf_emoticons_list = NULL;


static void emo_insert_emo (GtkTextBuffer *buffer, const gchar *text, gint len);


/* Ha! kf has now its own emoticon engine!!! */

/* This function inserts text into TextView, with emoticons and lkinks underlined... */
void emo_insert (GtkTextBuffer *buffer, const gchar *text) {
	gboolean done = FALSE;

	while (! done) {
		gchar *link, *end;
		GtkTextIter iter;

		link = strstr (text, "http://");
			/* Put all words before link */
		gtk_text_buffer_get_end_iter (buffer, &iter);
				
		//g_print ("DEBUG: link: %p, text: %p, link-text: %d\n", link, text, link-text);
		if (link)
			emo_insert_emo (buffer, text, link-text);
		else
			emo_insert_emo (buffer, text, strlen (text));
		
		if (link) {
			for (end = link; ! isspace (*end) && *end; end++)
				;
			gtk_text_buffer_get_end_iter (buffer, &iter);
			gtk_text_buffer_insert_with_tags_by_name (buffer, &iter,
					link, end-link, "link", NULL);
				
			if ((text = end) == '\0')
			done = TRUE;
		} else {
			done = TRUE;
		}
	}
}

/* This function inserts actually text into TextBuffer with emoticons...
 * to be called from above function */
static void emo_insert_emo (GtkTextBuffer *buffer, const gchar *text, gint len) {
	gboolean done = FALSE;

	//g_print ("Text to insert: %s, len %d\n", text, len);
	while (! done) {
		GList *tmp;
		gint first;
		KfEmoticon *femo = NULL;

		if (text == NULL)
			break;

		first = len;
		
		for (tmp = kf_emoticons_list; tmp; tmp=tmp->next) {
			KfEmoticon *emo = tmp->data;
			gchar *sub;
			gint pos;	/* Emo position */

			sub = strstr (text, emo->name);
			pos = sub - text;
			if (sub && pos <= len) {
				if (pos < first) {
					//first = MIN (first, pos);
					first = pos;
					femo = emo;
				}
			}
		}

		if (first > 0) {
			GtkTextIter iter;
//			g_print ("In: '%s'   (%d)\n",text, first);
			gtk_text_buffer_get_end_iter (buffer, &iter);
			gtk_text_buffer_insert (buffer, &iter, text, first);
			text += first;
			len -= first;
		}

		if (femo) {
			GtkTextIter iter;
			gint emo_len;
//			g_print ("In: '%s'   [emo]\n", femo->name);
			gtk_text_buffer_get_end_iter (buffer, &iter);
			gtk_text_buffer_insert_pixbuf (buffer, &iter, femo->pixbuf);
			emo_len = strlen (femo->name);
			text += emo_len;
			len -= emo_len;
		}

		if (len <= 0)
			done = TRUE;
	}
}


/*
 * Insetrs text into a GtkTextBuffer buffer with emoticons
 * Since i have absolutely no idea how to do this, I have adapted
 * code from GnuGadu2 (http://gadu.gnu.pl/)
 * by Marcin Krzyanowski and Igor Popik.
 * Thanks a lot!
 *
 * UPDATE: we don't use it anymore...
 */
void kf_emoticons_insert_text (GtkTextBuffer *buffer, const gchar *text) {
	GtkTextIter start_iter, end_iter;
	GtkTextMark *text_mark;
	GList *list;
	KfEmoticon *emot;

//	kf_emoticons_load ("emoticons.def");
	gtk_text_buffer_get_end_iter (buffer, &end_iter);
//	gtk_text_buffer_get_end_iter (buffer, &begin_of_inserted_text);
//	begin_of_inserted_text = end_iter;

	gtk_text_buffer_insert (buffer, &end_iter, text, -1);
	text_mark = gtk_text_buffer_get_insert (buffer);

	gtk_text_buffer_get_iter_at_mark (buffer, &start_iter, text_mark);
//	gtk_text_buffer_get_start_iter (buffer, &end_iter);
	gtk_text_buffer_get_end_iter (buffer, &end_iter);
//	gtk_text_iter_backward_char (&start_iter);

	for (list = kf_emoticons_list; list; list=list->next) {
		emot = list->data;
		while (gtk_text_iter_backward_search
				(&start_iter, emot->name, GTK_TEXT_SEARCH_VISIBLE_ONLY, &start_iter, &end_iter, NULL)) {

#ifdef ANIM
			GtkTextChildAnchor *anchor;
			GtkImage *img;

			g_print ("ANIM!!!\n");
			
			gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
			anchor = gtk_text_buffer_create_child_anchor (buffer, &start_iter);
			img = gtk_image_new_from_file (emot->path);
			gtk_text_view_add_child_at_anchor (buffer, img, anchor);
			gtk_widget_show (img);
			
#else
			gtk_text_buffer_delete (buffer, &start_iter, &end_iter);
			gtk_text_buffer_insert_pixbuf (buffer, &start_iter, emot->pixbuf);
#endif
			gtk_text_buffer_get_end_iter (buffer, &start_iter);
		}
	}
	
}

GList *kf_emoticons_load (const gchar *filename) {
	FILE *f;
	gchar line[1024];
	gchar *path;
	KfEmoticon *emot;

	path = g_path_get_dirname (filename);
	f = fopen (filename, "r");
	if (f == NULL)
		return kf_emoticons_list;
	
	while (fgets (line, 1024, f)) {
		gchar *e_name, *e_file, *tmp;
		e_name = line;

		for (tmp = line; *tmp != '\t' && *tmp != '\0' && *tmp != '\n'; tmp++)
			;
		if (*tmp == '\0' || *tmp == '\n')
			continue;
		else
			*tmp = '\0';
		
		/* Skip whitespace characters */
		++tmp;
		while (isspace (*tmp) && *tmp != '\0')
			tmp++;

		/* Get file name */
		e_file = tmp;

		for (; *tmp != '\0' && *tmp != '\n'; tmp++)
			;
		*tmp = '\0';

//		g_print ("Czytam emoticon %s (%s)... ", e_name, e_file);

		emot = kf_emoticon_new (e_name, path, e_file);
		if (emot) {
			kf_emoticons_list = g_list_append (kf_emoticons_list, emot);
		//	g_print ("ok\n");
		} else {
			//g_print ("failed to load emoticon %s (%s)\n", e_name, e_file);
		}
	}
	fclose (f);
	
	g_free (path);

	return kf_emoticons_list;
}

/* Creates a new Emoticon object */
KfEmoticon *kf_emoticon_new (const gchar *name, const gchar *path, const gchar *file) {
	gchar *fullpath;
	KfEmoticon *emot;
	GdkPixbuf *pixbuf;

	fullpath = g_build_filename (path, file, NULL);
	/* FIXME: I don't know whether to share a single GdkPixbuf among all
	 * with the same filename isn't a better idea; however it would
	 * need a hash of GdkPixbufs indexed by their filenames, thus it
	 * would slow all engine down... So now every kind of emoticon
	 * has its own GdkPixbuf.
	 */

#ifndef ANIM
	pixbuf = gdk_pixbuf_new_from_file (fullpath, NULL);
	g_free (fullpath);
	if (pixbuf == NULL)
		return NULL;
#else
	if (!g_file_test (fullpath, G_FILE_TEST_EXISTS))
		return NULL;
#endif

	emot = g_new (KfEmoticon, 1);
	emot->name = g_strdup (name);
	emot->filename = g_strdup (file);
	emot->pixbuf = pixbuf;

#ifdef ANIM
	emot->path = fullpath;
#endif
	

	return emot;
}

void kf_emoticons_init (void) {
	GList *tmp;
	KfEmoticon *emot;
	
	kf_emoticons_list = kf_emoticons_load (kf_find_file ("/emoticons/emoticons.def"));
	kf_emoticons_list = kf_emoticons_load (kf_config_file ("/emoticons/emoticons.def"));

	/* For debug purposes only */
//	for (tmp = kf_emoticons_list; tmp; tmp=tmp->next) {
//		emot = tmp->data;
//		g_print ("Zaladowano emoticon %s (%s)\n", emot->name, emot->filename);
//	}	
}

void kf_emo_init_textview (GtkTextView *tv) {
	GtkTextTag *url;
	GtkTextBuffer *buffer;

	buffer = gtk_text_view_get_buffer (tv);

	url = gtk_text_buffer_create_tag (buffer, "link",
			"foreground", kf_preferences_get_string ("colorChatUrl"),
			"underline", PANGO_UNDERLINE_SINGLE,
			NULL);

	g_signal_connect (G_OBJECT (tv), 
                      "button_press_event",
                      G_CALLBACK (textview_clicked_cb),
                      url);
	g_signal_connect (G_OBJECT (tv), 
                      "populate-popup",
                      G_CALLBACK (textview_popup_cb),
                      NULL);
}
	
/* Klikanie na ekranie... 
 *
 * We check if user clicked on a link... if so, we launch a browser:-)
 *
 * Or, if user clicked 3rd mouse button we store the URL
 * 
 * */
static gboolean textview_clicked_cb        (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data) {
	gint x, y;
	GtkTextIter iter;
	GtkTextTag *url = user_data;

	gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (widget), GTK_TEXT_WINDOW_WIDGET,
			event->x, event->y,
			&x, &y);
	gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (widget), &iter, x, y);

	if (gtk_text_iter_has_tag (&iter, url)) {
		GtkTextIter start, end;
		gchar *text;

		start = end = iter;

		while (! gtk_text_iter_begins_tag (&start, url))
			gtk_text_iter_backward_char (&start);

		while (! gtk_text_iter_ends_tag (&end, url))
			gtk_text_iter_forward_char (&end);

		text = gtk_text_iter_get_text (&start, &end);

		/* text is our link */
		if (event->button == 1) {
			kf_www_open (text);
		} else if (event->button == 3) {
			g_object_set_data_full (G_OBJECT (widget), "URL", g_strdup (text), g_free);
		}
		g_free (text);

		return FALSE;
	}

  return FALSE;
}

/*
 * Chceck if user clicked on an URL, then add "open link" entry to popup menu
 */
static void textview_popup_cb              (GtkTextView *textview,
                                            GtkMenu *menu,
                                            gpointer user_data) {
	gchar *text;

	if (text = g_object_get_data (G_OBJECT (textview), "URL")) {
		GtkWidget *item;

		item = gtk_separator_menu_item_new ();
		gtk_widget_show (item);
		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);

		item = gtk_menu_item_new_with_label (_("Open link"));
		gtk_widget_show (item);
		g_signal_connect (G_OBJECT (item), "activate",
				G_CALLBACK (open_link_cb), text);
		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
	}
}

static void open_link_cb                   (GtkMenuItem *menuitem,
                                            gpointer user_data) {
	kf_www_open (user_data);
}


