#include <gtk/gtk.h>
#include <glade/glade.h>
#include <time.h>
#include "kf.h"
#include "gui.h"
#include "jabber.h"
#include "events.h"
#include "chat.h"
#include "message.h"

/*
typedef struct {
	gchar *jid;
	GList *items;
} KfEventManager;
*/

/* Event structure */
typedef struct {
	gint id;
	gchar *jid;
	gchar *fulljid;
	KfEventClass klass;
	gpointer data;
	GDestroyNotify notify;
} KfEventElement;

/* Notify window */
static GtkWidget *event_box = NULL;
/* ID of timeout, that hides notify window */
static gint timeout_id = 0;

static GtkWidget *event_box_create (void);
static gboolean on_event_box_button_press  (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data);
static gboolean event_box_timeout (gpointer user_data);
static gchar *event_text (KfEventClass klass, gpointer user_data);
static gchar *event_text2 (KfEventClass klass, gpointer user_data);
static void kf_event_sound (KfEventClass klass);

/* Event queue */
static KfEventElement *kf_event_element_new (const gchar *jid, KfEventClass klass);
static void kf_event_element_free (KfEventElement *event);
static void kf_event_element_release (KfEventElement *event);

static GSList *event_queue = NULL;


void kf_event (KfEventClass klass, gpointer user_data) {
	GtkWidget *label;
	GtkWidget *label2;
	gint screen_width, screen_height;
	gint window_width, window_height;
	gchar *text;
	gchar *text2 = NULL;
	
	if (event_box == NULL)
		event_box = event_box_create ();

//	if (! kf_preferences_get_int ("enablePopups"))
//		return;

	label = g_object_get_data (G_OBJECT (event_box), "label");
	label2 = g_object_get_data (G_OBJECT (event_box), "label2");

	text = event_text (klass, user_data);
	/* FIXME - it doesn't work properly!!! */
	gtk_window_resize (GTK_WINDOW (event_box), 1, 1);
//	gtk_widget_set_size_request (event_box, 0, 0);
	gtk_label_set_text (GTK_LABEL (label), text);
	g_free (text);	

	if (kf_preferences_get_int ("enablePopups"))
		gtk_widget_show_all (event_box);

	text2 = event_text2 (klass, user_data);
	if (text2) {
		gtk_label_set_text (GTK_LABEL (label2), text2);
		g_free (text2);	
		gtk_widget_show (label2);
	} else {
		gtk_widget_hide (label2);
	}


	
	/* Place Notify window */
	screen_width = gdk_screen_width ();
	screen_height = gdk_screen_height ();
	gtk_window_get_size (GTK_WINDOW (event_box), &window_width, &window_height);
	gtk_window_move (GTK_WINDOW (event_box), screen_width - window_width - 16,
			screen_height - window_height - 16);
	gtk_window_get_size (GTK_WINDOW (event_box), &window_width, &window_height);
	gtk_window_move (GTK_WINDOW (event_box), screen_width - window_width - 16,
			screen_height - window_height - 16);

	/* Hide notify window in 10 seconds */
	if (timeout_id != 0)
		gtk_timeout_remove (timeout_id);
	timeout_id = gtk_timeout_add (10000, event_box_timeout, event_box);
}

static GtkWidget *event_box_create (void) {
	GtkWidget *window;	/* Window of a "tooltip" event window...*/
	GtkWidget *eventbox;	/* eeventbox */
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;	/* First label */
	GtkWidget *label2;	/* Secondary label */
	GtkWidget *image;	/* "Bulb" image */
	GdkColor postie = {0, 0xFFFF, 0xFFFF, 0xC0C0};
	GdkColor black = {0, 0x0, 0x0, 0x0};
	PangoFontDescription *font;

	window = gtk_window_new (GTK_WINDOW_POPUP);
//	gtk_widget_modify_bg (window, GTK_STATE_NORMAL, &black);

	eventbox = gtk_event_box_new ();
	gtk_widget_show (eventbox);
	gtk_container_add (GTK_CONTAINER (window), eventbox);
	
	g_signal_connect (G_OBJECT (eventbox), "button-press-event",
			G_CALLBACK (on_event_box_button_press), NULL);
//	gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &postie);
	gtk_widget_set_name (window, "gtk-tooltips");

	hbox = gtk_hbox_new (FALSE, 6);
	gtk_container_add (GTK_CONTAINER (eventbox), hbox);
	gtk_container_set_border_width (GTK_CONTAINER (eventbox), 1);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);

	image = gtk_image_new_from_file (kf_gui_find_file ("online.png"));
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);

	vbox = gtk_vbox_new (FALSE, 2);
	gtk_widget_show (vbox);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

	label = gtk_label_new ("blah");
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	label2 = gtk_label_new ("blah");
	gtk_widget_show (label2);
	font = pango_font_description_from_string ("italic 9");
	gtk_widget_modify_font (label2, font);
//	gtk_container_set_border_width (GTK_CONTAINER (eventbox), 1);
	gtk_box_pack_start (GTK_BOX (vbox), label2, FALSE, FALSE, 0);

	g_object_set_data (G_OBJECT (window), "label", label);
	g_object_set_data (G_OBJECT (window), "label2", label2);

	return window;
}

static gboolean on_event_box_button_press  (GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data) {
	gtk_widget_hide (gtk_widget_get_toplevel (widget));
}

static gboolean event_box_timeout (gpointer user_data) {
	gtk_widget_hide (user_data);
	return FALSE;
}

static gchar *event_text (KfEventClass klass, gpointer user_data) {
	gchar *text = NULL;
	if (klass == KF_EVENT_CLASS_MESSAGE) {
		KfJabberMessage *msg;
		KfJabberRosterItem *item;
		gchar *jid;

		msg = user_data;
		jid = g_strdup (msg->from);
		kf_jabber_jid_crop (jid);

		item = kf_jabber_roster_item_get (jid);
		text = g_strdup_printf (_("Message from %s"), item?item->display_name:msg->from);
		g_free (jid);

//		kf_sound_play ("bass.wav");
		kf_event_sound (KF_EVENT_CLASS_MESSAGE);
	} else if (klass == KF_EVENT_CLASS_CHAT) {
		KfJabberMessage *msg;
		KfJabberRosterItem *item;
		gchar *jid;

		msg = user_data;
		jid = g_strdup (msg->from);
		kf_jabber_jid_crop (jid);

		item = kf_jabber_roster_item_get (jid);
		text = g_strdup_printf (_("Chat with %s"), item?item->display_name:msg->from);
		g_free (jid);
//		kf_sound_play ("button.wav");
		kf_event_sound (KF_EVENT_CLASS_CHAT);
	} else if (klass == KF_EVENT_CLASS_PRESENCE) {
		KfJabberRosterItem *item;

		item = user_data;
		if (item) {
			time_t czas;
			struct tm *tajm;
		/*	g_print ("xxx\n");
			g_print ("1. %s\n", item->display_name);
			g_print ("2. %d\n", item->presence);
			g_print ("3. %s\n", kf_jabber_presence_type_to_string (item->presence));
			g_print ("4. %s\n", _(kf_jabber_presence_type_to_string (item->presence)));
			g_print ("5. %s\n", _("%s is now %s"));*/
			text = g_strdup_printf (_("%s is now %s"), item->display_name,
				_(kf_jabber_presence_type_to_string (item->presence)));
			czas = time (NULL);
			tajm = localtime (&czas);
			g_print (" * [%02d:%02d] %s\n", tajm->tm_hour, tajm->tm_min, text);

//			kf_sound_play ("harp.wav");
			if (item->presence != KF_JABBER_PRESENCE_TYPE_UNAVAILABLE) {
				kf_event_sound (KF_EVENT_CLASS_PRESENCE);
			} else {
				kf_event_sound (KF_EVENT_CLASS_OFFLINE);
			}
		} else {
//			g_print ("No item!");
			text = NULL;
		}
	}
	return text;
}

static gchar *event_text2 (KfEventClass klass, gpointer user_data) {
	gchar *text = NULL;
	if (klass == KF_EVENT_CLASS_MESSAGE) {
		KfJabberMessage *msg;
		gint len;

		msg = user_data;
		len = strlen (msg->body);
		if (len > 64) {
			text = g_strdup_printf ("%.63s...", msg->body);
		} else {
			text = g_strdup (msg->body);
		}		
	} else if (klass == KF_EVENT_CLASS_CHAT) {
		KfJabberMessage *msg;
		gint len;

		msg = user_data;
		len = strlen (msg->body);
		if (len > 160) {
			text = g_strdup_printf ("%.157s...", msg->body);
		} else {
			text = g_strdup (msg->body);
		}
	} else if (klass == KF_EVENT_CLASS_PRESENCE) {
		KfJabberRosterItem *item;

		item = user_data;
		if (item && item->status && *(item->status)) {
			text = g_strdup (item->status);
			g_print ("   %s\n", text);
		} else {
//			g_print ("No item!");
			text = NULL;
		}
	}
	return text;
}

/*
void kf_event_sound (const gchar *f) {
	gchar *cmd;

	cmd = g_strdup_printf ("play \"%s\" &", kf_find_file (f));
	system (cmd);
	g_free (cmd);
}*/

/* Kolejkowanie zdarzen */


/* Queues an event */
void kf_event_queue (KfEventClass klass, const gchar *jid, gpointer data, GDestroyNotify notify) {
	KfEventElement *event;

	event = kf_event_element_new (jid, klass);
	event->data = data;
	event->notify = notify;

	event_queue = g_slist_append (event_queue, event);
}

/* Release all events for jid */
void kf_event_release (const gchar *jid) {
	GSList *tmp;
	gchar *cropped_jid;
	cropped_jid = kf_jabber_jid_crop (g_strdup (jid));

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

		if (strcmp (cropped_jid, event->jid) == 0) {
			kf_event_element_release (event);
			event_queue = g_slist_delete_link (event_queue, tmp);
			kf_event_element_free (event);
		}
	}

	g_free (cropped_jid);
}

void kf_event_release_all (void) {
	GSList *tmp;

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

	//	if (strcmp (cropped_jid, event->jid) == 0) {
			kf_event_element_release (event);
			event_queue = g_slist_delete_link (event_queue, tmp);
			kf_event_element_free (event);
	//	}
	}
}

gboolean kf_event_get (const gchar *jid) {
	GSList *tmp;
	gchar *cropped_jid;
	cropped_jid = kf_jabber_jid_crop (g_strdup (jid));

	for (tmp = event_queue; tmp; tmp=tmp->next) {
		KfEventElement *event;

		event = tmp->data;

		if (strcmp (cropped_jid, event->jid) == 0) {
			g_free (cropped_jid);
			return TRUE;
		}
	}

	g_free (cropped_jid);
	return FALSE;
}

gboolean kf_event_get_all (void) {
	if (event_queue)
		return TRUE;
	else
		return FALSE;
}

static KfEventElement *kf_event_element_new (const gchar *jid, KfEventClass klass) {
	KfEventElement *event;

	event = g_new (KfEventElement, 1);
	event->jid = kf_jabber_jid_crop (g_strdup (jid));
	event->fulljid = g_strdup (jid);
	event->klass = klass;
	
	return event;
}
static void kf_event_element_free (KfEventElement *event) {
	g_free (event->jid);
	g_free (event->fulljid);
	g_free (event);
}

static void kf_event_element_release (KfEventElement *event) {
	if (event->klass == KF_EVENT_CLASS_MESSAGE) {
	} else if (event->klass == KF_EVENT_CLASS_CHAT) {
		kf_chat_window (event->fulljid);
	}
}
/*
static KfEventManager *kf_event_mnanager_new (void) {
	KfEventManager *self;

	self = g_new (KfEventManager, 1);
	*/

static void kf_event_sound (KfEventClass klass) {
	static gchar *ids[] = {"Msg", "Chat", "Presence", "", "Offline"};
	gchar *pref_name;

	pref_name = g_strdup_printf ("sound%sUse", ids[klass]);
	if (kf_preferences_get_int (pref_name)) {
		gchar *pref_file = g_strdup_printf ("sound%s", ids[klass]);

		kf_sound_play (kf_preferences_get_string (pref_file));
		g_free (pref_file);
	}
	g_free (pref_name);
}

