#include <gtk/gtk.h>
#include <glade/glade.h>
#include "kf.h"
#include "preferences.h"
#include "muc.h"
#include "muc_join.h"

static void join_cb (GtkButton *button, gpointer data);
static void use_passwd_toggled_cb          (GtkToggleButton *button,
                                            gpointer user_data);

void kf_muc_join_dialog (void) {
	GladeXML *glade;
	GtkWidget *dialog, *join, *cancel;
	GtkWidget *server, *room, *nick;
	GtkWidget *use_passwd, *passwd;

	glade = glade_xml_new (kf_find_file ("muc_join.glade"), NULL, NULL);
	kf_glade_get_widgets (glade,
			"join", &dialog, 
			"ok", &join,
			"cancel", &cancel,
			"server", &server,
			"room", &room,
			"nickname", &nick,
			"use_passwd", &use_passwd,
			"password", &passwd,
			NULL);
	
						/* TODO - check if we pass NULL here */
	gtk_entry_set_text (GTK_ENTRY (server), kf_preferences_get_string ("mucLastServer"));
	gtk_entry_set_text (GTK_ENTRY (room), kf_preferences_get_string ("mucLastRoom"));
	gtk_entry_set_text (GTK_ENTRY (nick), kf_preferences_get_string ("mucLastNick"));
	
	g_object_set_data_full (G_OBJECT (join), "Glade", glade, g_object_unref);
	g_signal_connect (G_OBJECT (join), "clicked", G_CALLBACK (join_cb), NULL);
	g_signal_connect (G_OBJECT (cancel), "clicked", G_CALLBACK (kf_destroy_toplevel), NULL);
	g_signal_connect (G_OBJECT (use_passwd), "toggled", G_CALLBACK (use_passwd_toggled_cb), passwd);
}

static void join_cb (GtkButton *button, gpointer data) {
	GladeXML *glade;
	GtkWidget *server_entry, *room_entry, *nick_entry, *use_passwd, *passwd_entry;
	const gchar *server, *room, *nick, *pass;
	gchar *jid;

	glade = glade_get_widget_tree (GTK_WIDGET (button));
	kf_glade_get_widgets (glade,
			"server", &server_entry,
			"room", &room_entry,
			"nickname", &nick_entry,
			"use_passwd", &use_passwd,
			"password", &passwd_entry,
			NULL);

	server = gtk_entry_get_text (GTK_ENTRY (server_entry));
	room = gtk_entry_get_text (GTK_ENTRY (room_entry));
	nick = gtk_entry_get_text (GTK_ENTRY (nick_entry));
	pass = gtk_entry_get_text (GTK_ENTRY (passwd_entry));

	kf_preferences_set ("mucLastServer", server);
	kf_preferences_set ("mucLastRoom", room);
	kf_preferences_set ("mucLastNick", nick);

	jid = g_strdup_printf ("%s@%s", room, server);
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (use_passwd))) {
		kf_muc_join_room_with_password (jid, nick, pass);
	} else {
		kf_muc_join_room (jid, nick);
	}
	g_free (jid);

	kf_destroy_toplevel (GTK_WIDGET (button), NULL);
}

static void use_passwd_toggled_cb          (GtkToggleButton *button,
                                            gpointer user_data)
{
	GtkWidget *entry = user_data;
	gtk_widget_set_sensitive (entry, gtk_toggle_button_get_active (button));
}
